%Simulation of spike-timing dependent plasticity (STDP) using integrate-and-fire
%neuron driven by two converging inputs


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Part One- generation of presynaptic spike trains by Carlos Brody
% This directory contains a number of files for generating and
% operating with sets of spike trains. The main file is
%
%    correlated_gen.m,
%
% which generates a set of Poisson spike trains that are pairwise
% correlated with each other; the number of cells, and amount and
% width of the correlation, are selected by the user.
%
% Run testgen to see an example of generation of correlated spikes.
% Or look in the documentation for correlated_gen.m
%
%
% SPIKE TRAIN FORMAT:
% ------------------
%
% All the files generate and operate spike trains in a format that
% is called (for very old historical reasons) Sillito-matrix
% format. The format of such a matrix S is as follows: 
%  - Each row represents one spike train. (Different rows could
%    then be different "trials" of one cell, or different cells
%    that were all "recorded" simultaneously in one trial.) 
%  - The first element (i.e., first column) of each row indicates
%    the number of spikes in that row. Thus S(:,1) is a column
%    vector with the number of spikes in each row.
%  - The second and onwards column elements each represent the
%    time of one spike, in ms. Thus S(2,3) will be the time of 
%    of the SECOND spike in the second row.
%
%    As an example, the matrix
%
%    [3  30 45.6 101 
%     1  55  0   0]
%
%    represents 2 spike trains, in the first of which there were 3
%    spikes, at 30, 45.6, and 101 ms; and in the second of which
%    there was only 1 spike, at 55 ms.
%
%
%
% ROUTINES AND PROGRAMS:
% ----------------------
%
% binner                Makes a binned PSTH
% correlated_gen        Generates pairwise-correlated spike trains
% dxcorr                Makes and plots shuffle-corrected x-corr
% nrastermaker          Plots rasters from one or two cells
% psth                  Makes a smoothed PSTH
% psthplot              Plots a smoothed PSTH
% uncorrelated_gen      Generates uncorrelated spike trains
% zxcorr                Does raw cross-correlograms of Sillito-format data

% Carlos Brody, Sep 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Part 2

%This directory contains IAF neuron with synaptic weights updated according to STDP rules, based on
%simulation of Song, Miller, Abbott 2000.

% ROUTINES AND PROGRAMS:
% ----------------------
% distribute_wInit        Distributes synaptic weights to slightly randomize initial synaptic weights
% iafstdp_8inhLev         Main file.  IAF with STDP learning rules driven by spike trains
%                         generated by correlated_gen 
% run_iafstdp_8inhLev     Calls iafstdp_6inhlev and initializes parameters values, user should set values here
% sillito_converter       Conversts Sillto-style spike trains generated by correlated_gen into 0 and 1s
% tutorial_iafstdp        Plots progression of synaptic weight changes,
%                        post synaptic firing rate, voltage trajector of
%                        simulation, inh and ex current, and inhib and ex
%                        conductance
% iafstdp                 Called by tutorial_iafstdp, equivelent to
%                           iaf_6inhLev, but with more outputs and fewer interations
% 
%Sandra Kuhlman Nov 2008





















