%binner    [psth, timebase] = binner(C)       generate binned psth
%
% C must be a Sillito-style matrix.
%
% [psth, timebase] = binner(C, {'from', from = 0}, 
%                              {'to', to = 2500},
%                              {'binwidth', binwidth = 6},
%                              ['plot' | 'noplot'],
%                              ['title' | 'notitle'])

function [ppsth, timebase] = binner(C, varargin)
    
    from = Inf;
    to = -Inf;
    do_title = 1;
    
    binwidth = 6;
    if nargout == 0, do_plot = 1;
    else do_plot = 0;
    end;
    
    % ------- parsing input args -------------
    
    arg = 1; while arg <= length(varargin),

       switch varargin{arg},
	  
	  case 'from',
	  if arg <= length(varargin), 
	     from = varargin{arg+1}; arg = arg + 1;
	  end;

	  case 'to',
	  if arg <= length(varargin), 
	     to = varargin{arg+1}; arg = arg + 1;
	  end;

	  case 'binwidth',
	  if arg <= length(varargin), 
	     binwidth = varargin{arg+1}; arg = arg + 1;
	  end;

	  case 'plot', do_plot = 1;
	  case {'no_plot', 'noplot'}, do_plot = 0;

	  case 'title', do_title = 1;
	  case {'no_title', 'notitle'}, do_title = 0;

	  otherwise,
	  varargin{arg},
	  error('Didn''t understand the above argument');
	  
       end;		
    arg = arg+1; end;
		    
    if to == -Inf, to = maxtime(C); end;
    if from == Inf, from = min([mintime(C), 0]); end;
    
    % ------ preparatory stuff  --------------
    
    
    timebase = from:binwidth:to;
    nbins = length(timebase);
    nrasters = rows(C);
    binwidth = ((to+1e-5)-from)/nbins;
    timebase = from:binwidth:to;
    
    % ------- the biz  -------------
    
    psth = zpsth(C, nbins, from, to);
    psth = psth / rows(C);
    
    if do_plot,
       stairize(timebase, (1000/binwidth)*psth);
	ylabel('Hz');
	xlabel('time (ms)');
	if do_title,
	   title(['binwidth = ' num2str(binwidth) ' ms']);
	end;
    end;
    
    if nargout > 0,
	ppsth = psth;
    end;
    



