

function [w_end,  p_end, m_end, v_end, w_MP, ExI_MP, InhI_MP, spike_count, nspike, params, v, gE, gI, stimes]...
         = iafstdp (T, prespikes_Tot, wInitialize, pInit, mInit, vInit, ifactor); 
  
%===========  all units in SI   =======================================================================================
   [mega, milli, nano,pico]=units;  %function returns prefix 10^x  

   dt=0.1*milli;          % time steps unit= seconds
   
   niter=(T*1000)*10;       %scalar indicating number of iterations, T is in units of seconds
   
   [Npre_total,Nprecol]=size(prespikes_Tot);  %Npre_total is total number of inputs, 80 is standard and use gmax=15*150*pico
   Npre=Npre_total/2;  %Npre is number of inputs in one set

   params= zeros(6,1);  % in order to output param values 
   

% --------------------------  model parameters  ---------------------------------


tau_syn_ex= 2*milli;               %scalar is decay time constant for ex synapses (Seconds)
tau_syn_inhib= tau_syn_ex*5;       %scalar is decay time constant for inhib synapses (Seconds)
gmax= 15*150*pico;                 %scalar is max conductance for individual presyn inputs (Siemens)  %Song used ~150 pS  
    params(1)=gmax; 
gmin=(.2*gmax)/50;                 %scalar is min conductance for individual presyn inputs (Siemens)
    params(2)= gmin;


E=  -65*milli;              %scalar is reversal potential for leak conductance  (Volts)
Ee=  0*milli;               %scalar is reversal potential for excitatory conductance (Volts)
Ei=  -70*milli;             %scalar is reversal potential for inhibitory conductance (Volts)

gL= 12.5*nano;	            %scalar is leak conductance (Siemens) % gL= 12.5 *nano; tau =20 ms
params(4)=gL;
c= (20*milli)/(1/gL);       %c=  tau_mem / (1/(gL));    (Farads)             
    params(3)= c;
threshIAF_Pyr=  -54*milli;  %is scalar for threshold for IAF  (Volts)
reset=  -65*milli;          %scalar resets Vm to this value after spike (Volts) 
refract=  20;               %20 iterations, equivilent to 2*10^6 Seconds
last_fire_time=  0;         %scalar  used to count refract period
spike=  0;                  %scalar used to indicate spike occured, 1 means spike at t+dt
spike_count=0;              %scalar to track firing rate of IAF Pyramid
        %------------- vectors
v= zeros (1,niter);             %1-D array membrane voltage
gE= zeros (1,niter);            %1-D array exctatory conducatnce feeds into voltage loop, updated by learning rule
gI= zeros (1,niter);            %1-D array inhibotiry conducatnce feeds into voltage loop, NOT updated by learning rule
gIPre= zeros(Npre_total,niter); %2-D array inhibitory conducatnce feeds into voltage loop, updated by learning rule
gPre= zeros(Npre*2,niter);      %2_D array columns are summed to fill gE 1-D array, not updated if learn =1.  Learn=1 is default.
Ex_current= zeros(1,niter);
Inh_current= zeros(1,niter);
 
        %--------- STDP learning rule vectors and parameters
p= zeros(Npre_total,niter);     %2-D array decay  for all pre_inputs, added to w at time t if spike 
m= zeros(1,niter);              %1-D array for calculating LTD decay
w= zeros(Npre_total,niter);     %2-D array synaptic weights of all nin pre inputs
wd= zeros(Npre_total,niter);    %2-D array for tracking synaptic weight changes, NOT necessary
wp= zeros(Npre_total,niter);    %2-D array for tracking synaptic weight changes, NOT necessary

wa= 0.0050;         %scalar is amount of increase if input preceedes spike  Song Miller Abbott: A-/A+=1.05, equivalent to 0.5% change of gmax
tau_p=20*milli;     %scalar is decay time constant used in p function for amount of potentiation for individual inputs (ms)
wb= -0.0045;        %scalar is amount of decrease if input follows spike  Song Miller Abbott: A-/A+=1.05
tau_m=35*milli;     %scalar is decay time constant used in m function for amount of depression for individual inputs (ms)



%------ More Points (mps), sample every 1000 iterations for graph outputs and calculations

spike_countMP=0;
nspike= zeros(1,10*T);
w_MP=  zeros(Npre_total,10*T);
wd_MP= zeros(Npre_total,10*T);      %NOT necessary, by default is not outputed, can add to output argument 
wp_MP= zeros(Npre_total,10*T);      %NOT necessary, by default is not outputed, can add to output argument 
ExI_MP= zeros(1,10*T);
InhI_MP= zeros(1,10*T);

%====================================   Inputs

%%%% generate ex - inhib sequence
t=1;
i=1;

shiftprespikes= zeros(Npre_total,niter);
     dc=100; %presynaptic time steps
     params(5)= 100/10; %convert delay to post syn time [unit= ms]
     d1=dc+1;   
  for i=1:Npre_total
  shiftprespikes (i,(find(prespikes_Tot(i,:))+dc))= 1;
  end
shiftprespikes=shiftprespikes(1:Npre_total,1:niter);

p(:,1)= pInit; %initilaizes from previous run
t=1;
i=1;
  for t=1:niter-1,  
    p(:,t+1)=p(:,t)*exp(-1*dt/tau_p)+ wa*gmax*(prespikes_Tot(:,t+1));  %all presynaptic threshold events decay by LTP function, is added to gPre if spike occurs
    gIPre(:,t+1)=gIPre(:,t)*exp(-1*dt/tau_syn_inhib)+ (ifactor*.088*gmax)*(shiftprespikes(:,t));
  end
  gI(1:niter)=sum(gIPre(:,:));
  
  
  randnoise= 0.1 *milli;
  params(6)= randnoise;  %multiplied by randn function to generate random fluctuation in v
 
%%%%%%%%%%%%%%%%%%%%%%%%  leaky IAF loop %%%%%%%%%%%%%%%

  %because looping SongSTDPepsc_ipsc.m, need to carry over previous values from last value 
   t=1;
   v(1,1)= vInit;
   w(1:Npre_total,1)=wInitialize;
   m(1,1)= mInit;
    
 spike=0;
 stimes= zeros(1,1);
 %=========================================== IAF with STDP learning  
   while t < niter, 
    gE(t)=sum(gPre(:,t));
    
    v(t+1) = v(t) + dt*(1/c)*((E-v(t))*gL+ (Ee-v(t))*gE(t)+ (Ei-v(t))*gI(t))+ randnoise*randn; 
    Ex_current(t)= (Ee-v(t))*gE(t);  % to output total ex current
    Inh_current(t)=(Ei-v(t))*gI(t);  % to output total inh current
    
         if v(t)>=threshIAF_Pyr,
             last_fire_time=t+1;
             v(t)=40*milli;
             spike= 1;         spike_count= spike_count+1;  spike_countMP= spike_countMP + 1;
             if spike_count==1, stimes=t; else stimes(length(stimes)+1)=t; end
         else spike=0;
         end    
    if (t-last_fire_time)<refract, v(t+1)=reset;end

       
     m(t+1)=m(t)*exp(-1*dt/tau_m) + wb*gmax*spike;    %if spike, LTD decay curve calculated, to be subtracted from the w of any ipresynaptic nput that fires
     w(:,t+1) = w(:,t) + p(:,t)*spike + m(t)*(prespikes_Tot(:,t+1)); %synaptic weight for each individual pre input is updated based on m function and if spike occured
     wd(:,t+1) = wd(:,t)  + m(t+1)*(prespikes_Tot(:,t+1));  % tracking depression
     wp(:,t+1) = wp(:,t) + p(:,t)*spike;   %tracking potentiation use +p if want to include LTP effect  

     gPre(:,t+1)=gPre(:,t)*exp(-1*dt/tau_syn_ex)+ (w(:,t+1)).*(prespikes_Tot(:,t+1)); 

       max_ceiling= find(w(:,t+1)>gmax);  %generates column      %w is reset to gmax if exceed gmax
         if max_ceiling~=0, w(max_ceiling,t+1)=gmax;end;
         min_ceiling= find(w(:,t+1)<gmin);
         if min_ceiling~=0, w(min_ceiling,t+1)=gmin;end;
     
   t=t+1;  
  end
%==============================================  IAF loop finished 


%----------------------------------- output  variables

w_end= w(:,niter-1);
p_end= p(:,niter-1);
m_end=m(niter);
v_end=v(niter-1);

%-------------------More Points (mps)in 100 ms bins 

     for i= 1:T*10-1,
       p= 1000*i;
       nspike(i)= length(find (v(p:p+1000) >30*milli));
       w_MP(:,i)= w(:,(i*1000));
       wd_MP(:,i)= wd(:,(i*1000));
       wp_MP(:,i)= wp(:,(i*1000));
       ExI_MP (:,i)= Ex_current(:,(i*1000));
       InhI_MP (:,i)= Inh_current(:,(i*1000));
     end
     
   nspike(1)=  length(find (v(1:1000) >30*milli));  
   w_MP(:,T*10)= w(:,(niter-1));
   wd_MP(:,T*10)= wd(:,(niter-1));
   wp_MP(:,T*10)= wp(:,(niter-1));   
   ExI_MP (:,T*10)= Ex_current(:,(niter-1));
   InhI_MP (:,T*10)= Inh_current(:,(niter-1));
     


