% iaf model with STDP learning, based on Song, Miller, and Abbott 2000.
% Primary output: synaptic weight value for each presynaptic input: w_end.
% Most parameter values can be varied using run_iafstdp M file.
%Can implement history dependence by uncommetting, if do so, comment line
%160.  To implement a change in gain, use iafstdp_8inhLev_varyItau_gain
%rather than this function.

function [w_end, p_end, m_end, v_end, params]=...
         iafstdp_8inhLev (T, prespikes_Tot, wInitialize, pInit, mInit, vInit, ifactor) 

     %all units in SI
[mega, milli, nano,pico]=units;
dt=0.1*milli;          % unit= seconds
niter=(T*1000)*10;       %scalar indicating number of steps 
[Npre_total,Nprecol]=size(prespikes_Tot);
Npre=Npre_total/2;

iTau=5.75;
tau_syn_ex= 2*milli;               %scalar is decay time constant for ex synapses 
tau_syn_inhib= iTau*milli;            %changed sep 22 2008 was tau_syn_ex*5 %scalar is decay timeconstant for inhib synapses %changed 05927 was 4
% tau inh young= 5.75, old= 4.20
% to match total conductance of one ex event and one inh event:
% Ex 2ms: 1.14864  if .249*gmax ...  Inh 5.75ms: 1.1479  if .088*gmax


%params= zeros(1,6);  % in order to output param values cparams= char( 'capacitance','gmax','Ints','gL','gL_Int', 'delay','aRI', 'RI freq'); 

%----------------
E=  -65*milli;              %scalar is reversal potential (mV)
Ee=  0*milli;               %scalar is reversal potential for excitatory conductance 
Ei=  -70*milli;             %scalar is reversal potential for inhibitory conductance 

gL= 12.5*nano;	                %scalar is leak conductance 1/Rin siemens % gL= 12.5 *nano*adj; tau =20 ms
params(4)=gL;
c= (20*milli)/(1/gL);
%c= 2.50E-05;                    %c=  tau_mem / (1/(gL));
params(3)= c;
threshIAF_Pyr=  -54*milli;  %is scalar for threshold for IAF  %-54
reset=  -65*milli;          %scalar resets Vm to this value after spike  (mV)
refract=  20;       %20 iterations, equivilent to 2 ms
last_fire_time=  0;             %scalar  used to count refract period
spike=  0;                      %scalar used to indicate spike occured, 1 means spike at t+dt
spiketimes=0;

v= zeros (1,niter);  %1-D array membrane voltage
gE= zeros (1,niter); %1-D array exctatory conducatnce feeds into voltage loop, updated by learning rule
gRI= zeros(1,niter); %1-D array random inhibition conductance feeds into voltage loop defult is to not use
gIPre= zeros(Npre_total,niter);
gIPre_gn= zeros(Npre_total,niter);
gPre= zeros(Npre*2,niter);    %2_D array columns are summed to fill gE 1-D array, not updated if learn =1.  Learn=1 is default.
Ex_charge= zeros(1,niter);
Inh_charge= zeros(1,niter);
spike_count=0;        %scalar to track firing rate of IAF Pyramid
gmax= 15*150*pico;        %scalar is max conductance for presyn input  %Song 2000 used ~150 pS  
params(1)=gmax;  

hstrydependance= .25; %LTP-related-only history, if potentiated once at full (0.5%), next time will be (.4%)
hstryd_factor= zeros(Npre_total,1);
p= zeros(Npre_total,niter);%potential LTP, 2-D array decay  for all pre_inputs, added to w at time t if spike 
m= zeros(1,niter);   %1-D array for calculating LTD decay
w= zeros(Npre_total,niter); %synaptic weights of all nin pre inputs
wa= .005;            %scalar is amount of increase if input preceedes spike  Song Miller Abbott: A-/A+=1.05
tau_p=20*milli;         %scalar is decay time constant used in p function for amount of potentiation for individual inputs (ms)
wb= -.0045;         %scalar is amount of decrease if input follows spike  Song Miller Abbott: A-/A+=1.05
tau_m=35*milli;         %scalar is decay time constant used in m function for amount of depression for individual inputs (ms)
gmin=(.2*gmax)/50;               %scalar
params(2)= gmin;

gI= zeros(1,niter);

wd= zeros(Npre_total,niter);  %2-D array for calculating synaptic weight changes
wp= zeros(Npre_total,niter);  %2-D array for calculating synaptic weight changes
wa= .005;            %scalar is amount (amplitude) of increase if input preceedes spike  Song Miller Abbott: A-/A+=1.05



%====================================   Inputs


        [Npre_total,Nprecol] = size(prespikes_Tot);
        shiftprespikes= zeros(Npre_total,Nprecol);
       
     
           
             
             
            
                
            %%------------------- adding in Inh conductance by row
         i=1;       
         for i=1:Npre_total
             jit= randperm(60) + 40;
             shiftprespikes (i,(find(prespikes_Tot(i,:)))+ jit(1) )= 1;
             
             
         end


         shiftprespikes= shiftprespikes(1:Npre_total,1:Nprecol);
        
        
         
%-----------------------------------------------------

p(:,1)= pInit; %initilaizes from previous run

  for t=1:niter-1,  
    p(:,t+1)=p(:,t)*exp(-1*dt/tau_p)+ wa*gmax*(prespikes_Tot(:,t+1));  %all presynaptic threshold events decay by LTP function, is added to gPre if spike occurs
    gIPre(:,t+1)=gIPre(:,t)*exp(-1*dt/tau_syn_inhib)+ (ifactor*.088*gmax)*(shiftprespikes(:,t)); % inhibitory conductance generated here

  end
  gI(1:niter)=sum(gIPre(:,:));
 
  
  gI= gI(1:niter);
  
  randnoise= 0.1 *milli;
  params(6)= randnoise;
 
%%%%%%%%%%%%%%%%%%%%%%%%  LIAF loop %%%%%%%%%%%%%%%

  %because looping iafstdp.m, need to carry over previous values from last value (generally T=2, or 20,0000 iterations) 
   t=1;
   v(1,1)= vInit;
   w(1:Npre_total,1)=wInitialize;
   m(1,1)= mInit;
    
 spike=0;
 stimes= zeros(1,1);
 %=========================================== IAF   
   while t < niter, 
    gE(t)=sum(gPre(:,t));
    
    v(t+1) = v(t) + dt*(1/c)*((E-v(t))*gL+ (Ee-v(t))*gE(t)+ (Ei-v(t))*gI(t))+ randnoise*randn;
    
    %Ex_charge(t)= (Ee-v(t))*gE(t);  % use to calculate total ex charge
    %Inh_charge(t)=(Ei-v(t))*gI(t);  % use to calculate total inh charge
    
         if v(t)>=threshIAF_Pyr,
             last_fire_time=t+1;
             v(t)=40*milli;
             spike= 1;         spike_count= spike_count+1;  
             if spike_count==1, stimes=t; else stimes(length(stimes)+1)=t; end  %stimes is total spike number
         else spike=0;
         end    
    if (t-last_fire_time)<refract, v(t+1)=reset;end

       
    
     %%%%%%%% Uncomment if want histroy dependence %%%%%%%%%%%%%%%%%%%%
     %---- adding history dependence for LTP
%      wp(:,t+1) = wp(:,t) + p(:,t)*spike;   %tracking LTP only
%         if spike==1,  
%             hstryd_factor= 1-(wp(:,t+1).*hstrydependance);               
%         else hstryd_factor(:,1)=1;    
%         end
     %---------------------------------------
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        

     m(t+1)=m(t)*exp(-1*dt/tau_m) + wb*gmax*spike;    %if spike, LTD decay curve calculated, to be subtracted from the w of any ipresynaptic nput that fires
     w(:,t+1) = w(:,t) + p(:,t)*spike + m(t)*(prespikes_Tot(:,t+1)); %use if no history dependance, comment thsi line if using hist dep 
     %Uncomment one line below if want hist dep
     %w(:,t+1) = w(:,t) + p(:,t)*spike .*hstryd_factor + m(t)*(prespikes_Tot(:,t+1)); %use if want history dependance
    
     gPre(:,t+1)=gPre(:,t)*exp(-1*dt/tau_syn_ex)+ (w(:,t+1)).*(prespikes_Tot(:,t+1)); 

       max_ceiling= find(w(:,t+1)>gmax);  %generates column      %w is reset to gmax if exceed gmax
         if max_ceiling~=0, w(max_ceiling,t+1)=gmax;end;
         min_ceiling= find(w(:,t+1)<gmin);
         if min_ceiling~=0, w(min_ceiling,t+1)=gmin;end;
     
   t=t+1;  
  end
%==============================================  IAF loop finished 

w_end= w(:,niter-1);
p_end= p(:,niter-1);  m_end=m(niter); v_end=v(niter-1);
