% maxtime	maxtime(C)	Find maximum spike time for Sillito C-matrix
%
%  [max, rasternum] = maxtime(C, [start=1], [end=size(C,1)])
%
%  Takes a cell matrix, as generated by rasterize, and returns the least
%  spiking time for the whole set of rasters. max is that time; rasternum
%  is the raster number which had that early-firing spike.
%
%  If there is a tie, the first raster number is reported.
%

function [maxt, rasternum] = maxtime(C, start, eend)

	if nargin == 1,  start = 1; eend = size(C,1); end;
	if nargin == 2,  eend = size(C,1); end;

	if start < 1, start = 1; end;
	if eend > size(C,1), eend = size(C,1); end;
	
	rasternum = 0;
	maxt = 0;

	for i=start:eend
	   if C(i,1) > 0
	      maxt = C(i, C(i,1)+1);
	      rasternum = i;
	      break;
	   end;
	end;
	
	for i=i+1:eend,
	   if C(i,1) > 0  &  C(i, C(i,1)+1) > maxt
	      maxt = C(i, C(i,1)+1);
	      rasternum = i;
	   end;
	end;
	

