% mintime	mintime(C)	Find minimum spike time for Sillito C-matrix
%
%  [min, rasternum] = mintime(C, [start=1], [end=size(C,1)])
%
%  Takes a cell matrix, as generated by rasterize, and returns the least
%  spiking time for the whole set of rasters. min is that time; rasternum
%  is the raster number which had that early-firing spike.
%
%  If there is a tie, the first raster number is reported.
%

function [mint, rasternum] = mintime(C, start, eend)

	if nargin == 1,  start = 1; eend = size(C,1); end;
	if nargin == 2,  eend = size(C,1); end;

	if start < 1, start = 1; end;
	if eend > size(C,1), eend = size(C,1); end;
	
	rasternum = 0;
	mint = 0;

	for i=start:eend
		if C(i,1) > 0
		   mint = C(i, 2);
		   rasternum = i;
		   break;
		end;
	end;

	for i=i+1:eend,
		if C(i,1) > 0  &  C(i, 2) < mint
		   mint = C(i,2);
		   rasternum = i;
		end;
	end;

