%multisum     [a] = multisum(v, u, s)
%
%  v must be a vector, u a vector of indices into v, and s either
%  a  scalar or a vector the same length as u. Calling this
%  function is equivalent to doing.  
%
%     a = v; a(u) = v(u) + s;
%
%  BUT if u contains repeated indices, s is summed repeatedly. 
%  Thus,
%
%     a = multisum([0 0 0], [1 1 2], 2)            = [4 2 0];
%
%  WHEREAS
%
%     a = [0 0 0]; a([1 2 2]) = a([1 2 2]) + 2     = [2 2 0];
%
%  BECAUSE normal matlab doesn't do the repeated sum, which is 
%  sometimes convenient. 
%

% Implemented as a mex function, multisum.c.
% Carlos Brody June 1998


function [a] = multisum(v, u, s)
   
   a = v; if length(s)==1, s = ones(1, length(u))*s; end;
   for i=1:length(u),
      a(u(i)) = a(u(i)) + s(i);
   end;
   