% psth (C)		Make psth from a cell
%
%	[histo, timebase] = psth(C, [sdev=6], [dt=0.1])
%
% Returns a vector that is a smoothed PSTH. C must be a Sillito
% cell matrix, as those produced by rasterize. sdev is the
% standard deviation, in ms, of the Gaussian used to smooth the
% resulting PSTH. 
%
% timebase is a vector such that plot(timebase, psth) results in
% an adequate x-axis. dt between bins (and for the timebase) is by
% default 0.1. psth is in units of counts per bin. 
%

function [hhisto, ttimebase] = psth(C, sdev, dt, from, to)

   if nargin<5, to   = -Inf; end;
   if nargin<4, from =  Inf; end;
        
   if nargin < 3, dt = 0.1; end;
    
   if nargin < 2, sdev = 6; end;
    
    [histo, timebase] = binner(C, 'binwidth', dt, 'noplot', ...
	'from', from, 'to', to);
    histo = smooth([0 histo 0], sdev/dt);
    histo = histo(2:end-1);
    % This zero-padding to ensure that smooth.m doesn't pad with
    % spikes! 
    
    if min(histo) < 0,
	if abs(min(histo)) > 1e-10, 
	    fprintf(2, 'psth.m: HACK WARNING!\n');
	end;
	
	histo = histo - min(histo);
    end;
    
    if nargout == 0,
	plot(timebase, histo*(1000/dt));
	xlabel('time (ms)');
	ylabel('firing rate (Hz)');
    else
	hhisto = histo; % *(1000/dt);
	ttimebase = timebase;
    end;


