% psthplot.m		Make and plot PSTH's
%
%	[] = psthplot(C1, [C2], [sdev=6])
%
% C1 and the optional argument C2 must be cell Sillito matrices, such as those
% prodcued by rasterize. sdev is the standard deviation of the Gaussian
% used to smooth the PSTH's.
%
% The plot is produced on the current figure. The order of C2 and sdev is
% not important, either one, both, or none may be present in either order.
%

function [] = psthplot(C1, arg2, arg3)

	if nargin == 1
		[histo, timebase] = psth(C1, 6);
		plot(timebase, histo*10000);
		xlabel('(ms)');
		ylabel('(Hz)');
		return;
	end;

	if nargin == 2  &  isscalar(arg2)
		[histo, timebase] = psth(C1, arg2);
		plot(timebase, histo*10000);
		xlabel('(ms)');
		ylabel('(Hz)');
		return;
	else
		sdev = 6;
		C2 = arg2;
	end;
		
	if nargin == 3
		if isscalar(arg2); sdev = arg2;	C2 = arg3;
		elseif isscalar(arg3); sdev = arg3; C2 = arg2;	end;
	end;

	[h1, t1] = psth(C1, sdev);
	[h2, t2] = psth(C2, sdev);
		
	plot(t1, h1*10000, t2, h2*10000);
	xlabel('(ms)');
	ylabel('(Hz)');


