%uncorrelated_gen.m  S = uncorrelated_gen(T, N, rate)
% 
% Makes a matrix of spike times, in "Sillito" format. Each row is
% uncorrelated with any of the others. Each row is made up of
% Poisson spikes at a uniform rate. Note that spike times are in
% milliseconds, but T and rate are in secs and spikes/sec.
%
% RETURNS:
% --------
%
%  -S     A matrix in "Sillito" format (see Contents.m)
%
%
% PARAMETERS:
% -----------
%
%  -T     total time, in secs, for which to generate spikes.
%
%  -N     number of spike trains to generate
%
%  -rate  spikes/sec for generating spikes.
%
%
% EXAMPLES:
% ---------
%
%  generate 20 spike trains for 2 seconds at 30 spikes/sec; then
%  plot a raster:
% 
%  >> S = uncorrelated_gen(2, 20, 30); nrastermaker(S)
%


% written by Carlos Brody, Sep 2004


function [S] = uncorrelated_gen(T, N, rate)
   
   S = [poissrnd(rate*T, N, 1) ...
	  zeros(N, ceil(rate*T + 4*sqrt(rate*T)))];
   
   for i=1:rows(S),
      spiketimes = rand(1, S(i,1))*T*1000;
      S(i,2:length(spiketimes)+1) = sort(spiketimes);
   end;
   S = S(:,1:1+max(S(:,1)));


